@@ -8,6 +8,8 @@
 #' @include escape.R
 #' @include sql.R
 #' @include utils.R
+#' @include compat-obj-type.R
+#' @include compat-types-check.R
 NULL
 
 #' @export
@@ -162,9 +164,11 @@ base_scalar <- sql_translator(
   ifelse = function(test, yes, no) sql_if(enquo(test), enquo(yes), enquo(no)),
 
   switch = function(x, ...) sql_switch(x, ...),
-  case_when = function(...) sql_case_when(...),
+  case_when = function(..., .default = NULL, .ptype = NULL, .size = NULL) {
+    sql_case_when(..., .default = .default, .ptype = .ptype, .size = .size)
+  },
+  case_match = sql_case_match,
 
-  sql = function(...) sql(...),
   `(` = function(x) {
     sql_expr(((!!x)))
   },
@@ -193,7 +197,9 @@ base_scalar <- sql_translator(
   # Impala - https://impala.apache.org/docs/build/html/topics/impala_bigint.html
   as.integer64  = sql_cast("BIGINT"),
 
-  c = function(...) c(...),
+  c = function(...) {
+    c(...)
+  },
   `:` = function(from, to) from:to,
 
   between = function(x, left, right) {
@@ -219,9 +225,9 @@ base_scalar <- sql_translator(
   month = function(x) sql_expr(EXTRACT(month %from% !!x)),
   day = function(x) sql_expr(EXTRACT(day %from% !!x)),
   mday = function(x) sql_expr(EXTRACT(day %from% !!x)),
-  yday = sql_not_supported("yday()"),
-  qday = sql_not_supported("qday()"),
-  wday = sql_not_supported("wday()"),
+  yday = sql_not_supported("yday"),
+  qday = sql_not_supported("qday"),
+  wday = sql_not_supported("wday"),
   hour = function(x) sql_expr(EXTRACT(hour %from% !!x)),
   minute = function(x) sql_expr(EXTRACT(minute %from% !!x)),
   second = function(x) sql_expr(EXTRACT(second %from% !!x)),
@@ -254,38 +260,45 @@ base_scalar <- sql_translator(
   str_trim = sql_str_trim,
   str_c = sql_paste(""),
   str_sub = sql_str_sub("SUBSTR"),
+  str_like = function(string, pattern, ignore_case = TRUE) {
+    if (isTRUE(ignore_case)) {
+      sql_expr(!!string %LIKE% !!pattern)
+    } else {
+      cli::cli_abort("Backend only supports case insensitve {.fn str_like}.")
+    }
+  },
 
-  str_conv = sql_not_supported("str_conv()"),
-  str_count = sql_not_supported("str_count()"),
-  str_detect = sql_not_supported("str_detect()"),
-  str_dup = sql_not_supported("str_dup()"),
-  str_extract = sql_not_supported("str_extract()"),
-  str_extract_all = sql_not_supported("str_extract_all()"),
-  str_flatten = sql_not_supported("str_flatten()"),
-  str_glue = sql_not_supported("str_glue()"),
-  str_glue_data = sql_not_supported("str_glue_data()"),
-  str_interp = sql_not_supported("str_interp()"),
-  str_locate = sql_not_supported("str_locate()"),
-  str_locate_all = sql_not_supported("str_locate_all()"),
-  str_match = sql_not_supported("str_match()"),
-  str_match_all = sql_not_supported("str_match_all()"),
-  str_order = sql_not_supported("str_order()"),
-  str_pad = sql_not_supported("str_pad()"),
-  str_remove = sql_not_supported("str_remove()"),
-  str_remove_all = sql_not_supported("str_remove_all()"),
-  str_replace = sql_not_supported("str_replace()"),
-  str_replace_all = sql_not_supported("str_replace_all()"),
-  str_replace_na = sql_not_supported("str_replace_na()"),
-  str_sort = sql_not_supported("str_sort()"),
-  str_split = sql_not_supported("str_split()"),
-  str_split_fixed = sql_not_supported("str_split_fixed()"),
-  str_squish = sql_not_supported("str_squish()"),
-  str_subset = sql_not_supported("str_subset()"),
-  str_trunc = sql_not_supported("str_trunc()"),
-  str_view = sql_not_supported("str_view()"),
-  str_view_all = sql_not_supported("str_view_all()"),
-  str_which = sql_not_supported("str_which()"),
-  str_wrap = sql_not_supported("str_wrap()")
+  str_conv = sql_not_supported("str_conv"),
+  str_count = sql_not_supported("str_count"),
+  str_detect = sql_not_supported("str_detect"),
+  str_dup = sql_not_supported("str_dup"),
+  str_extract = sql_not_supported("str_extract"),
+  str_extract_all = sql_not_supported("str_extract_all"),
+  str_flatten = sql_not_supported("str_flatten"),
+  str_glue = sql_not_supported("str_glue"),
+  str_glue_data = sql_not_supported("str_glue_data"),
+  str_interp = sql_not_supported("str_interp"),
+  str_locate = sql_not_supported("str_locate"),
+  str_locate_all = sql_not_supported("str_locate_all"),
+  str_match = sql_not_supported("str_match"),
+  str_match_all = sql_not_supported("str_match_all"),
+  str_order = sql_not_supported("str_order"),
+  str_pad = sql_not_supported("str_pad"),
+  str_remove = sql_not_supported("str_remove"),
+  str_remove_all = sql_not_supported("str_remove_all"),
+  str_replace = sql_not_supported("str_replace"),
+  str_replace_all = sql_not_supported("str_replace_all"),
+  str_replace_na = sql_not_supported("str_replace_na"),
+  str_sort = sql_not_supported("str_sort"),
+  str_split = sql_not_supported("str_split"),
+  str_split_fixed = sql_not_supported("str_split_fixed"),
+  str_squish = sql_not_supported("str_squish"),
+  str_subset = sql_not_supported("str_subset"),
+  str_trunc = sql_not_supported("str_trunc"),
+  str_view = sql_not_supported("str_view"),
+  str_view_all = sql_not_supported("str_view_all"),
+  str_which = sql_not_supported("str_which"),
+  str_wrap = sql_not_supported("str_wrap")
 )
 
 base_symbols <- sql_translator(
@@ -320,15 +333,19 @@ base_agg <- sql_translator(
   all        = sql_aggregate("MIN"),
   any        = sql_aggregate("MAX"),
 
-  sd         = sql_not_supported("sd()"),
-  var        = sql_not_supported("var()"),
-  cor        = sql_not_supported("cor()"),
-  cov        = sql_not_supported("cov()"),
+  sd         = sql_not_supported("sd"),
+  var        = sql_not_supported("var"),
+  cor        = sql_not_supported("cor"),
+  cov        = sql_not_supported("cov"),
 
   # Ordered set functions
   quantile = sql_quantile("PERCENTILE_CONT", "ordered"),
   median = sql_median("PERCENTILE_CONT", "ordered"),
 
+  # `first()`, `last()` and `nth()` don't work as aggregate functions in SQL
+  # b/c grouping takes places before sorting => it is undefined which value comes
+  # first/last. See
+  # https://stackoverflow.com/a/46805009/7529482
   # first = sql_prefix("FIRST_VALUE", 1),
   # last = sql_prefix("LAST_VALUE", 1),
   # nth = sql_prefix("NTH_VALUE", 2),
@@ -371,7 +388,7 @@ base_win <- sql_translator(
       sql_expr(LAST_VALUE(!!x)),
       win_current_group(),
       order_by %||% win_current_order(),
-      win_current_frame()
+      win_current_frame() %||% c(-Inf, Inf)
     )
   },
   nth = function(x, n, order_by = NULL) {
@@ -409,10 +426,10 @@ base_win <- sql_translator(
   all   = win_aggregate("MIN"),
   any   = win_aggregate("MAX"),
 
-  sd    = sql_not_supported("sd()"),
-  var   = sql_not_supported("var()"),
-  cor   = sql_not_supported("cor()"),
-  cov   = sql_not_supported("cov()"),
+  sd    = sql_not_supported("sd"),
+  var   = sql_not_supported("var"),
+  cor   = sql_not_supported("cor"),
+  cov   = sql_not_supported("cov"),
 
   # Ordered set functions
   quantile = sql_quantile("PERCENTILE_CONT", "ordered", window = TRUE),
@@ -492,4 +509,4 @@ sql_random.DBIConnection <- function(con) {
   sql_expr(RANDOM())
 }
 
-utils::globalVariables("RANDOM")
+utils::globalVariables(c("RANDOM", "%LIKE%"))
