@@ -5,17 +5,19 @@ select_query <- function(from,
                          where = character(),
                          group_by = character(),
                          having = character(),
+                         window = character(),
                          order_by = character(),
                          limit = NULL,
                          distinct = FALSE) {
 
-  stopifnot(is.character(select))
-  stopifnot(is.character(where))
-  stopifnot(is.character(group_by))
-  stopifnot(is.character(having))
-  stopifnot(is.character(order_by))
-  stopifnot(is.null(limit) || (is.numeric(limit) && length(limit) == 1L))
-  stopifnot(is.logical(distinct), length(distinct) == 1L)
+  check_character(select)
+  check_character(where)
+  check_character(group_by)
+  check_character(having)
+  check_character(window)
+  check_character(order_by)
+  check_number_whole_inf(limit, allow_null = TRUE)
+  check_bool(distinct)
 
   structure(
     list(
@@ -24,6 +26,7 @@ select_query <- function(from,
       where = where,
       group_by = group_by,
       having = having,
+      window = window,
       order_by = order_by,
       distinct = distinct,
       limit = limit
@@ -45,30 +48,39 @@ print.select_query <- function(x, ...) {
   if (length(x$select))   cat("Select:   ", named_commas(x$select), "\n", sep = "")
   if (length(x$where))    cat("Where:    ", named_commas(x$where), "\n", sep = "")
   if (length(x$group_by)) cat("Group by: ", named_commas(x$group_by), "\n", sep = "")
+  if (length(x$window))   cat("Window:   ", named_commas(x$window), "\n", sep = "")
   if (length(x$order_by)) cat("Order by: ", named_commas(x$order_by), "\n", sep = "")
-  if (length(x$having))   cat("Having:   ", named_commas(x$having), "\n", sep = "")
+  if (length(x$having))   cat("Having:   ", named_commas(x$having), "\n", sep = "") # nocov
   if (length(x$limit))    cat("Limit:    ", x$limit, "\n", sep = "")
 }
 
 #' @export
-sql_optimise.select_query <- function(x, con = NULL, ...) {
+sql_optimise.select_query <- function(x, con = NULL, ..., subquery = FALSE) {
   if (!inherits(x$from, "select_query")) {
     return(x)
   }
 
-  from <- sql_optimise(x$from)
+  from <- sql_optimise(x$from, subquery = subquery)
 
   # If all outer clauses are executed after the inner clauses, we
   # can drop them down a level
-  outer <- select_query_clauses(x)
-  inner <- select_query_clauses(from)
+  outer <- select_query_clauses(x, subquery = subquery)
+  inner <- select_query_clauses(from, subquery = TRUE)
 
   can_squash <- length(outer) == 0 || length(inner) == 0 || min(outer) > max(inner)
 
   if (can_squash) {
+    # Have we optimised away an ORDER BY
+    if (length(from$order_by) > 0 && length(x$order_by) > 0) {
+      if (!identical(x$order_by, from$order_by)) {
+        warn_drop_order_by()
+      }
+    }
+
     from[as.character(outer)] <- x[as.character(outer)]
     from
   } else {
+    x$from <- from
     x
   }
 }
@@ -77,14 +89,15 @@ sql_optimise.select_query <- function(x, con = NULL, ...) {
 # https://sqlbolt.com/lesson/select_queries_order_of_execution
 
 # List clauses used by a query, in the order they are executed in
-select_query_clauses <- function(x) {
+select_query_clauses <- function(x, subquery = FALSE) {
   present <- c(
     where =    length(x$where) > 0,
     group_by = length(x$group_by) > 0,
     having =   length(x$having) > 0,
-    select =   !identical(x$select, sql("*")),
+    select =   !identical(unname(x$select), sql("*")),
     distinct = x$distinct,
-    order_by = length(x$order_by) > 0,
+    window = length(x$window) > 0,
+    order_by = (!subquery || !is.null(x$limit)) && length(x$order_by) > 0,
     limit    = !is.null(x$limit)
   )
 
@@ -92,46 +105,38 @@ select_query_clauses <- function(x) {
 }
 
 #' @export
-sql_render.select_query <- function(query, con, ..., bare_identifier_ok = FALSE) {
-  from <- sql_subquery(con,
-    sql_render(query$from, con, ..., bare_identifier_ok = TRUE),
-    name = NULL
+sql_render.select_query <- function(query, con, ..., subquery = FALSE, lvl = 0) {
+  from <- dbplyr_sql_subquery(con,
+    sql_render(query$from, con, ..., subquery = TRUE, lvl = lvl + 1),
+    name = NULL,
+    lvl = lvl
   )
 
-  sql_select(
+  dbplyr_query_select(
     con, query$select, from,
     where = query$where,
     group_by = query$group_by,
     having = query$having,
+    window = query$window,
     order_by = query$order_by,
     limit = query$limit,
     distinct = query$distinct,
     ...,
-    bare_identifier_ok = bare_identifier_ok
+    subquery = subquery,
+    lvl = lvl
   )
 }
 
-# SQL generation ----------------------------------------------------------
-
-#' @export
-sql_select.DBIConnection <- function(con, select, from, where = NULL,
-                               group_by = NULL, having = NULL,
-                               order_by = NULL,
-                               limit = NULL,
-                               distinct = FALSE,
-                               ...) {
-  out <- vector("list", 7)
-  names(out) <- c("select", "from", "where", "group_by", "having", "order_by",
-    "limit")
-
-  out$select    <- sql_clause_select(select, con, distinct)
-  out$from      <- sql_clause_from(from, con)
-  out$where     <- sql_clause_where(where, con)
-  out$group_by  <- sql_clause_group_by(group_by, con)
-  out$having    <- sql_clause_having(having, con)
-  out$order_by  <- sql_clause_order_by(order_by, con)
-  out$limit     <- sql_clause_limit(limit, con)
-
-  escape(unname(purrr::compact(out)), collapse = "\n", parens = FALSE, con = con)
+warn_drop_order_by <- function() {
+  # Rules according to SQLite: https://sqlite.org/forum/forumpost/878ca7a9be0862af?t=h
+  # 1. There is no LIMIT clause in the subquery
+  # 3. The subquery is not part of the FROM clause in an UPDATE-FROM statement
+  # 4. The outer query does not use any aggregate functions other than the built-in count(), min(), and/or max() functions.
+  # 5. Either the outer query has its own ORDER BY clause or else the subquery is one term of a join.
+  #
+  # (2. and 6. are left out as they are SQLite internal)
+  warn(c(
+    "ORDER BY is ignored in subqueries without LIMIT",
+    i = "Do you need to move arrange() later in the pipeline or use window_order() instead?"
+  ))
 }
-
