@@ -34,8 +34,13 @@ sql_clause <- function(kw, parts, sep = ",", parens = FALSE, lvl = 0) {
   clause
 }
 
-sql_clause_select <- function(con, select, distinct = FALSE, top = NULL, lvl = 0) {
-  assert_that(is.character(select))
+sql_clause_select <- function(con,
+                              select,
+                              distinct = FALSE,
+                              top = NULL,
+                              lvl = 0,
+                              call = caller_env()) {
+  check_character(select, call = call)
   if (is_empty(select)) {
     cli_abort("Query contains no columns")
   }
@@ -54,12 +59,12 @@ sql_clause_from  <- function(from, lvl = 0) {
   sql_clause("FROM", from, lvl = lvl)
 }
 
-sql_clause_where <- function(where, lvl = 0) {
+sql_clause_where <- function(where, lvl = 0, call = caller_env()) {
   if (length(where) == 0L) {
     return()
   }
 
-  assert_that(is.character(where))
+  check_character(where, call = call)
   where_paren <- sql(paste0("(", where, ")"))
   sql_clause("WHERE", where_paren, sep = " AND", lvl = lvl)
 }
@@ -68,12 +73,12 @@ sql_clause_group_by <- function(group_by, lvl = 0) {
   sql_clause("GROUP BY", group_by)
 }
 
-sql_clause_having <- function(having, lvl = 0) {
+sql_clause_having <- function(having, lvl = 0, call = caller_env()) {
   if (length(having) == 0L) {
     return()
   }
 
-  assert_that(is.character(having))
+  check_character(having, call = call)
   having_paren <- sql(paste0("(", having, ")"))
   sql_clause("HAVING", having_paren, sep = " AND")
 }
